package me.timeto.shared

import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.update
import me.timeto.shared.db.TaskFolderDb
import me.timeto.shared.db.TaskFolderDb.Companion.sortedFolders
import me.timeto.shared.db.TaskDb
import me.timeto.shared.TextFeatures.TimeData
import me.timeto.shared.vm.__VM
import me.timeto.shared.vm.ui.sortedByFolder

class WatchTabTasksVM : __VM<WatchTabTasksVM.State>() {

    class TaskUI(
        val task: TaskDb,
    ) {

        val textFeatures = task.text.textFeatures()
        val listText = textFeatures.textUi()

        val timeUI: TimeUI? = textFeatures.timeData?.let { timeData ->
            val unixTime = timeData.unixTime
            val timeLeftText = timeData.timeLeftText()
            val daytimeText = daytimeToString(unixTime.time - unixTime.localDayStartTime())
            val textColor = when (timeData.status) {
                TimeData.STATUS.IN -> ColorRgba.textSecondary
                TimeData.STATUS.SOON -> ColorRgba.blue
                TimeData.STATUS.OVERDUE -> ColorRgba.red
            }
            TimeUI(
                text = "$daytimeText  $timeLeftText",
                textColor = textColor,
            )
        }

        fun start(
            onStarted: () -> Unit,
            needSheet: () -> Unit, // todo data for sheet
        ) {
            val autostartData = taskAutostartData(task) ?: return needSheet()
            launchExDefault {
                WatchToIosSync.startTaskWithLocal(
                    activity = autostartData.first,
                    timer = autostartData.second,
                    task = task,
                )
                onStarted()
            }
        }

        class TimeUI(
            val text: String,
            val textColor: ColorRgba,
        )
    }

    data class FolderUI(
        val title: String,
        val tasks: List<TaskUI>,
    )

    data class State(
        val foldersUI: List<FolderUI>
    )

    override val state = MutableStateFlow(
        State(
            foldersUI = listOf()
        )
    )

    override fun onAppear() {
        val scope = scopeVM()
        TaskDb.getAscFlow().onEachExIn(scope) { tasks ->
            upFolders(tasks)
        }
        TaskFolderDb.anyChangeFlow().onEachExIn(scope) {
            upFolders(TaskDb.getAsc())
        }
    }

    private suspend fun upFolders(allTasks: List<TaskDb>) {
        val foldersUI = TaskFolderDb.getAscBySort().sortedFolders().map { folder ->
            FolderUI(
                title = folder.name,
                tasks = allTasks
                    .filter { it.folder_id == folder.id }
                    .sortedByFolder(folder)
                    .map { TaskUI(it) }
            )
        }
        state.update { it.copy(foldersUI = foldersUI) }
    }
}
